@@ -86,9 +86,6 @@ static const struct fuse_opt fuse_mount_opts[] = {
 	FUSE_DUAL_OPT_KEY("neglect_shares",	KEY_KERN),
 	FUSE_DUAL_OPT_KEY("push_symlinks_in",	KEY_KERN),
 	FUSE_OPT_KEY("nosync_unmount",		KEY_KERN),
-#if __FreeBSD_version >= 1200519
-	FUSE_DUAL_OPT_KEY("intr",		KEY_KERN),
-#endif
 	/* stock FBSD mountopt parsing routine lets anything be negated... */
 	/*
 	 * Linux specific mount options, but let just the mount util
